/**
 * Removes properties from an object, based on the values in an array, and returns the new object.
 * Equivalent to an object version of TS Omit<>
 */
export const omit = <
  A extends Record<PropertyKey, unknown>,
  const B extends ReadonlyArray<PropertyKey>
>(
  objToPluck: A,
  keysToPluck: B | (keyof A)[]
): Omit<A, B[number]> =>
  Object.keys(objToPluck)
    .filter(
      (key) => !keysToPluck.map((el) => (typeof el === 'string' ? el : el.toString())).includes(key)
    )
    .reduce((result, key) => ({...result, [key]: objToPluck[key]}), {} as Omit<A, B[number]>)

/**
 * Picks properties from an object, base on the values in an array, and returns the new object.
 * Equivalent to an object version of TS Pick<>
 */
export const pick = <
  A extends Record<PropertyKey, unknown>,
  const B extends ReadonlyArray<PropertyKey>
>(
  objToPluck: A,
  keysToPluck: B | (keyof A)[]
): Pick<A, B[number]> =>
  [...keysToPluck].reduce((memo, prop) => {
    memo[prop] = objToPluck[prop]
    return memo
  }, {} as Record<PropertyKey, unknown>) as Pick<A, B[number]>
